# pylint: disable=missing-docstring
import os
import re

from . import CONFIG


def is_matching_file(f):
    files, patterns = CONFIG.exclude_files
    files_match = (f.endswith(".py") or f.endswith(".pyx")) and f not in files
    patterns_match = all(re.search(p, f) is None for p in patterns)
    return files_match and patterns_match


def is_matching_dir(d):
    dirs, patterns = CONFIG.exclude_pkgs
    dirs_match = d not in dirs
    patterns_match = all(re.search(p, d) is None for p in patterns)
    return dirs_match and patterns_match


def get_all_file_mappings():
    doc_api_path = os.path.join(CONFIG.docs, "api")
    if not os.path.isdir(doc_api_path):
        os.mkdir(doc_api_path)

    whole_walk = list(os.walk(CONFIG.source))
    src_walk = list(filter(lambda x: "__pycache__" not in x[0], whole_walk))

    # stores the source sub-package directory : docs sub-package file mappings
    subpkg_doc_files = {
        CONFIG.source: os.path.join(doc_api_path, CONFIG.package_name + ".rst")
    }
    # stores the source file : docs file mappings
    submod_doc_files = {}
    for root, dirs, files in src_walk:
        dd = os.path.normpath(root).split(os.sep)
        pkg_idx = dd.index(CONFIG.package_name)
        if CONFIG.package_name in dd[(pkg_idx + 1) :]:
            pkg_idx += dd[(pkg_idx + 1) :].index(CONFIG.package_name)
        subpkg_unfolded = dd[pkg_idx:]

        try:
            dirs.remove("__pycache__")
        except ValueError:
            pass

        for d in dirs:
            if not is_matching_dir(d):
                continue

            spkg_name = ".".join(subpkg_unfolded) + (".%s" % d) + ".rst"

            doc_real_path = doc_api_path
            if len(subpkg_unfolded) > 1:
                for sub in subpkg_unfolded[1:]:
                    doc_real_path = os.path.join(doc_real_path, sub)

            subpkg_doc_files[os.path.join(root, d)] = os.path.normpath(
                os.path.join(doc_real_path, spkg_name)
            )

        for f in files:
            if not is_matching_file(f) or not is_matching_dir(dd[-1]):
                continue

            # form the name of the corresponding doc file
            fname = f.split(".")[0]
            src_file = os.path.normpath(os.path.join(root, f))

            submod_name = ".".join(subpkg_unfolded) + (".%s" % fname) + ".rst"

            doc_real_path = doc_api_path
            if len(subpkg_unfolded) > 1:
                for sub in subpkg_unfolded[1:]:
                    doc_real_path = os.path.join(doc_real_path, sub)

            doc_file = os.path.join(doc_real_path, submod_name)

            submod_doc_files[src_file] = doc_file

    return subpkg_doc_files, submod_doc_files


def file_mappings(sources: list):
    subpkg_doc_files, submod_doc_files = get_all_file_mappings()
    mappings = {
        source: submod_doc_files[source]
        for source in sources
        if source in submod_doc_files
    }

    return subpkg_doc_files, mappings


def mapping_tostr(mapping: dict):
    s = ""
    for k, v in mapping.items():
        s += f"\n    {k} -> {v}"
    return s



import inspect
import os

TEMPLATES_DIR = os.path.join(
    os.path.dirname(
        os.path.abspath(inspect.getframeinfo(inspect.currentframe()).filename)
    ),
    "templates",
)

_VALID_MATCHES = list(filter(lambda x: x.endswith(".rst"), os.listdir(TEMPLATES_DIR)))


def load(name):
    if not name.endswith(".rst"):
        name += ".rst"

    if name not in _VALID_MATCHES:
        raise ValueError("Unrecognised template: %s" % name)

    with open(os.path.join(TEMPLATES_DIR, name)) as f:
        contents = f.read()

    return contents


def _indented_object_block(objects):
    block = "\n    ".join(objects)
    return block


def _header_break(name):
    return "".join(["=" for _ in range(len("``" + name + "``"))])


def make_subpkg_overview(name, submodules):
    header_break = _header_break(name)
    if submodules:
        split = name.split(".")
        if len(split) == 1:
            dirname = None
        else:
            dirname = split[-1]

        submodules_summaries = make_submodule_summaries(submodules, dirname,)
    else:
        submodules_summaries = ""

    return load("subpkg_overview").format(
        header_break=header_break, name=name, submodules=submodules_summaries
    )


def make_submodule_summaries(submodules, dirname=None):
    submod_block = _indented_object_block(submodules)
    if dirname is not None:
        toctree = ":toctree: {}/".format(dirname)
    else:
        toctree = ":toctree: ."

    return load("submodules_summaries").format(toctree=toctree, submodules=submod_block)


def make_classes_overview(dirname, classes):
    classes_block = _indented_object_block(classes)
    return load("classes_overview").format(dirname=dirname, classes=classes_block)


def make_dataclasses_overview(dirname, dcs):
    dataclasses_block = _indented_object_block(dcs)
    return load("dataclasses_overview").format(dirname=dirname, dcs=dataclasses_block)


def make_enums_overview(dirname, enums):
    enums_block = _indented_object_block(enums)
    return load("enums_overview").format(dirname=dirname, enums=enums_block)


def make_single_class_overview(name):
    return load("single_class_overview").format(name=name)


def make_functions_section(functions):
    functions_block = _indented_object_block(functions)
    return load("functions_section").format(functions=functions_block)


def make_properties_section(cls_name, properties):
    properties = ["%s.%s" % (cls_name, p) for p in properties]
    properties_block = _indented_object_block(properties)
    return load("properties_section").format(properties=properties_block)


def make_methods_section(cls_name, methods, constructor=None):
    methods = ["%s.%s" % (cls_name, m) for m in methods]
    methods_block = _indented_object_block(methods)

    if constructor is not None:
        constructor_block = "\n    %s" % constructor
    else:
        constructor_block = ""

    return load("methods_section").format(
        constructor=constructor_block, methods=methods_block
    )


def make_simplemod_header(name, modname):
    header_break = _header_break(name)

    return load("simplemod_header").format(
        header_break=header_break, name=name, module_name=modname,
    )


def make_singleobj_header(name, modname, obj_name, props, methods, constr):
    header_break = _header_break(name)

    obj_overview = make_single_class_overview(obj_name)

    if props:
        prop_section = make_properties_section(obj_name, props)
    else:
        prop_section = ""

    if methods:
        if constr is not None:
            constructor = "%s.%s" % (obj_name, constr)
        else:
            constructor = None

        meth_section = make_methods_section(obj_name, methods, constructor)
    else:
        meth_section = ""

    return load("singleclass_header").format(
        header_break=header_break,
        name=name,
        module_name=modname,
        overview=obj_overview,
        properties=prop_section,
        methods=meth_section,
    )


def make_multiclass_header(
    name: str,
    modname: str,
    subdir_name: str,
    classes: list,
    dataclasses: list,
    enums: list,
    functions: list,
):
    header_break = _header_break(name)

    if classes:
        classes_overview = make_classes_overview(subdir_name, classes)
    else:
        classes_overview = ""

    if dataclasses:
        dcs_overview = make_dataclasses_overview(subdir_name, dataclasses)
    else:
        dcs_overview = ""

    if enums:
        enums_overview = make_enums_overview(subdir_name, enums)
    else:
        enums_overview = ""

    if functions:
        functions_section = make_functions_section(functions)
    else:
        functions_section = ""

    return load("multiclass_header").format(
        header_break=header_break,
        name=name,
        module_name=modname,
        classes=classes_overview,
        dataclasses=dcs_overview,
        enums=enums_overview,
        functions=functions_section,
    )



import os
import re
import sys

from . import templates
from . import CONFIG

from .loader import is_matching_file, is_matching_dir
from .parsing import SourceFileInfo


def write(subpkg_doc_files: dict, submod_doc_files: dict):
    for subpkg_dir, subpkg_doc_file in subpkg_doc_files.items():
        name = subpkg_doc_file.split(os.sep)[-1].replace(".rst", "")
        all_items = os.listdir(subpkg_dir)
        submodules = _filter_submodules(all_items, subpkg_dir)

        contents = templates.load.make_subpkg_overview(name, submodules)

        leading_dir, _ = os.path.split(subpkg_doc_file)
        os.makedirs(leading_dir, exist_ok=True)
        with open(subpkg_doc_file, "w+") as f:
            f.write(_format_writable(contents))

    for sfi, doc_file in submod_doc_files.items():
        _manipulate_doc_file(doc_file, sfi)


def _filter_submodules(items, subpkg_dir):
    submods = []

    for item in items:
        if item == "__pycache__":
            continue

        path = os.path.join(subpkg_dir, item)
        dd = os.path.normpath(path).split(os.sep)
        pkg_idx = dd.index(CONFIG.package_name)
        if CONFIG.package_name in dd[(pkg_idx + 1) :]:
            pkg_idx += dd[(pkg_idx + 1) :].index(CONFIG.package_name)
        subpkg_unfolded = dd[pkg_idx:]

        if os.path.isdir(path) and is_matching_dir(item):
            sub_items = os.listdir(path)
            if "__init__.py" in sub_items:
                submods.append(".".join(subpkg_unfolded))
        else:
            if is_matching_file(item):
                if subpkg_unfolded[-1].endswith(".py"):
                    subpkg_unfolded[-1] = subpkg_unfolded[-1].replace(".py", "")
                elif subpkg_unfolded[-1].endswith(".pyx"):
                    subpkg_unfolded[-1] = subpkg_unfolded[-1].replace(".pyx", "")

                submods.append(".".join(subpkg_unfolded))

    submods.sort()
    return submods


def _format_writable(contents: str):
    contents = contents.strip()
    return re.sub(r"\n\s*\n", "\n\n", contents) + "\n"


def _manipulate_doc_file(doc_file: str, sfi: SourceFileInfo):
    # recursively make all directories up to doc_file if they don't exist already
    os.makedirs(os.path.dirname(doc_file), exist_ok=True)

    public_objs = sfi.public()
    if not public_objs and not sfi.public("functions"):
        _, fname = os.path.split(doc_file)
        name = fname.replace(".rst", "")
        contents = templates.load.make_simplemod_header(name, name)
        with open(doc_file, "w+") as f:
            f.write(_format_writable(contents))
    else:
        _multiobj_handler(doc_file, sfi, public_objs)


def _multiobj_handler(doc_file: str, sfi: SourceFileInfo, objs: dict):
    leading_dir, fname = os.path.split(doc_file)
    new_dirname = fname.split(".")[-2]

    # Write the module level doc file first
    name = fname.replace(".rst", "")
    classes = list(objs.get("classes", {}).keys())
    dataclasses = list(objs.get("dataclasses", {}).keys())
    enums = list(objs.get("enums", {}).keys())
    functions = list(sfi.public("functions").keys())

    contents = templates.load.make_multiclass_header(
        name, name, new_dirname, classes, dataclasses, enums, functions,
    )
    with open(doc_file, "w+") as f:
        f.write(_format_writable(contents))

    # Now write all the sub doc files from all the objects
    # that exist within this module - creating a directory
    # with module name if it doesn't already exist
    full_new_dirname = os.path.join(leading_dir, new_dirname)
    if not os.path.isdir(full_new_dirname):
        os.mkdir(full_new_dirname)

    for obj_type, objects in objs.items():
        for obj in objects:  # NOTE: obj is the object name
            new_fname = ".".join(fname.split(".")[:-1]) + (".%s.rst" % obj)
            fpath = os.path.join(leading_dir, os.path.join(new_dirname, new_fname))

            name = new_fname.replace(".rst", "")
            mod_name = fname.replace(".rst", "")

            if obj_type == "classes":
                cd = sfi.get_class_data(obj)

                constructor = None if cd.constructor is None else cd.constructor[0]
                properties = list(cd.properties.keys())
                methods = list(cd.methods.keys())
            else:
                constructor = None
                properties = []
                methods = []

            contents = templates.load.make_singleobj_header(
                name, mod_name, obj, properties, methods, constructor,
            )

            with open(fpath, "w+") as f:
                f.write(_format_writable(contents))



import os

from . import CONFIG


def clean_api_docs():
    doc_api_path = os.path.join(CONFIG.docs, "api")
    if not os.path.isdir(doc_api_path):
        return

    walk = list(os.walk(doc_api_path))
    for root, dirs, files in walk:
        for f in files:
            af = os.path.abspath(os.path.join(root, f))

            if f == "index.rst":  # don't remove index file(s)
                continue

            if f.endswith(".rst"):
                os.remove(af)

    for root, _, _ in walk:
        if root == doc_api_path:
            continue

        try:
            os.rmdir(root)
        except OSError:
            pass



from collections import OrderedDict
from inspect import isclass
import re

from . import object_info as oi
from . import CONFIG

from .enums import MethodType


class SourceFileInfo(object):
    def __init__(self, src):
        self.src = src
        # TODO might be better to store all these as dicts
        #      of mapping - name : ObjectInfo
        self.classes = OrderedDict()
        self.enums = OrderedDict()
        self.dataclasses = OrderedDict()
        self.methods = {}  # {classname : {methname : MethodInfo}}
        self.functions = OrderedDict()

    def public(self, objtype=None, include_functions=False):
        if objtype is None:
            public = {}

            pclasses = self.public("classes")
            if pclasses:
                public["classes"] = pclasses

            penums = self.public("enums")
            if penums:
                public["enums"] = penums

            pdcs = self.public("dataclasses")
            if pdcs:
                public["dataclasses"] = pdcs

            if include_functions:
                pfuncs = self.public("functions")
                if pfuncs:
                    public["functions"] = pfuncs

            return public

        objects = getattr(self, objtype)

        return {k: v for k, v in objects.items() if v.is_public}

    def get_class_data(self, classname: str):
        if classname not in self.classes:
            raise ValueError("%s not in self.classes" % classname)

        all_methods = self.methods.get(classname, {})
        methods = {}
        properties = {}
        constructor = None
        for method_name, method_info in all_methods.items():
            if method_info.mtype == MethodType.METHOD and method_info.is_public:
                methods[method_name] = method_info
            elif method_info.mtype == MethodType.PROPERTY and method_info.is_public:
                properties[method_name] = method_info
            elif method_info.mtype == MethodType.CONSTRUCTOR:
                if constructor is not None:
                    raise Exception(
                        "BUG: class %s somehow has multiple constructors." % classname
                    )
                constructor = method_name, method_info

        return ClassData(classname, methods, constructor, properties)


class ClassData(object):
    def __init__(self, name, methods=None, constructor=None, properties=None):
        self.name = name
        self.methods = methods
        self.properties = properties
        self.constructor = constructor


def strip_base_args_whitespace(line):
    if "(" not in line or ")" not in line:
        return line.split(":")[0]

    start_paren_idx = line.index("(")
    end_paren_idx = line.index(")")

    bases = line[(1 + start_paren_idx) : end_paren_idx]
    reformed_bases = bases.replace(" ", "")

    return line.split("(")[0] + f"({reformed_bases})"


def is_class(line):
    return line.startswith("class") or line.startswith("cdef class")


def is_method(line, stripped):
    return (
        (stripped.startswith("def") or stripped.startswith("cpdef"))
        and "(" in line
        and len(line) - len(stripped) == 4
    )


def is_free_function(line):
    return (line.startswith("def") or line.startswith("cpdef")) and "(" in line


class Parser(object):
    def __init__(self, src):
        self.src = src
        self.sfi = SourceFileInfo(self.src)
        self.line = ""
        self.prev_line = ""

    def _parse_class(self, streampos):
        line = strip_base_args_whitespace(self.line)

        split = line.split(" ")
        if "cdef" in line:
            if len(split) > 3:
                split = split[:3]
            _, _, name_with_bases = split
        else:
            if len(split) > 2:
                split = split[:2]
            _, name_with_bases = split

        tmp = name_with_bases.split("(")
        if len(tmp) > 1:
            bases = tmp[1]
        else:
            bases = ""
        name = tmp[0].split(":")[0]

        if "Enum" in bases or "Flag" in bases:
            if all(re.search(p, name) is None for p in CONFIG.exclude_enums):
                self.sfi.enums[name] = oi.EnumInfo(name, self.src, streampos)
        elif "dataclass" in self.prev_line:
            if all(re.search(p, name) is None for p in CONFIG.exclude_dataclasses):
                self.sfi.dataclasses[name] = oi.DataClassInfo(name, self.src, streampos)
        else:
            if all(re.search(p, name) is None for p in CONFIG.exclude_classes):
                self.sfi.classes[name] = oi.ClassInfo(
                    name, self.src, streampos, "ABC" in bases
                )

    def _parse_free_function(self, streampos):
        # cases to consider:
        # 1) def foo():
        # 2) def foo(arg):
        # 3) def foo(arg1, arg2,):
        # 4) def foo(
        #       arg1, arg2,
        #    ):
        # 5) def foo(arg1,
        #       arg2,):
        # 6) def foo(arg1,
        #       arg2,
        #    ):
        # and all of the above with optional "arg : type" for typing support (along
        # with optional "-> type" after function declaration)
        # and (for Cython) all of the above with cpdef and optional type before foo

        # 4, 5 and 6 will require multi-line parsing so need to pass a handle
        # to the IO file stream to this function
        line = strip_base_args_whitespace(self.line)

        split = line.split(" ")
        if "cpdef" in line:
            if len(split) == 2:
                _, name_with_args = split
            else:
                if len(split) > 3:
                    split = split[:3]
                _, _type, name_with_args = split
        else:
            if len(split) > 2:
                split = split[:2]
            _, name_with_args = split

        tmp = name_with_args.split("(")
        name = tmp[0]

        if all(re.search(p, name) is None for p in CONFIG.exclude_functions):
            self.sfi.functions[name] = oi.FunctionInfo(name, self.src, streampos)

    def _parse_methods(self, src_handle):
        class_info_list = list(self.sfi.classes.values())
        for i, (classname, class_info) in enumerate(self.sfi.classes.items()):
            src_handle.seek(class_info.streampos)

            self.sfi.methods[classname] = OrderedDict()

            if class_info.streampos == class_info_list[-1].streampos:
                allclass = src_handle.read()
            else:
                allclass = src_handle.read(
                    class_info_list[i + 1].streampos - class_info.streampos
                )

            class_lines = allclass.splitlines()
            for j, line in enumerate(class_lines):
                if is_free_function(line):  # Broken outside of class now so leave
                    break

                stripped = line.strip()

                if is_method(line, stripped):
                    line = strip_base_args_whitespace(stripped)

                    line = line.split("(")[0]
                    split = line.split(" ")
                    if "cpdef" in line:
                        if len(split) == 2:
                            _, name_with_args = split
                        else:
                            if len(split) == 3:
                                _, _type, name_with_args = split
                            elif len(split) == 4:
                                _, _kw, _type, name_with_args = split
                            else:
                                raise RuntimeError(
                                    f"Encountered unparsable method signature: {line}"
                                )
                    else:
                        if len(split) > 2:
                            split = split[:2]
                        _, name_with_args = split

                    tmp = name_with_args.split("(")
                    method_name = tmp[0]

                    if any(
                        re.search(p, method_name) is not None
                        for p in CONFIG.exclude_methods
                    ):
                        continue

                    is_property = class_lines[j - 1].lstrip().startswith("@property")
                    is_constructor = method_name == "__init__"

                    if is_property:
                        mtype = MethodType.PROPERTY
                    elif is_constructor:
                        mtype = MethodType.CONSTRUCTOR
                    else:
                        mtype = MethodType.METHOD

                    self.sfi.methods[classname][method_name] = oi.MethodInfo(
                        method_name, self.src, src_handle.tell(), mtype, class_info
                    )

    def parse(self):
        with open(self.src, encoding="utf8") as f:
            self.line = f.readline()

            in_docstrings = False
            while self.line:
                if not in_docstrings:
                    # Inside a module-level docstring...
                    if self.line.startswith('"""') or self.line.startswith('r"""'):
                        # ... check to see if it's multi-line
                        in_docstrings = self.line.count('"""') == 1
                    else:
                        if is_class(self.line):
                            self._parse_class(f.tell())
                        elif is_free_function(self.line):
                            self._parse_free_function(f.tell())
                else:  # Still inside a multi-line module-level docstring
                    # so check to see if it's ended
                    in_docstrings = not self.line.endswith('"""\n')

                self.prev_line = self.line
                self.line = f.readline()

            self._parse_methods(f)

        return self.sfi



import os
import re

try:
    from .version import version as __version__
except ImportError:
    __version__ = "?.?.?"


def info_print(msg, verbose_flag):
    if verbose_flag:
        print("[reslate] " + msg)


class Config(object):
    def __init__(self):
        self._psp = None
        self._pkgname = None
        self._dsp = None

        self._exclude_files = [], []
        self._exclude_pkgs = [], []
        self._exclude_classes = []
        self._exclude_enums = []
        self._exclude_dataclasses = []
        self._exclude_methods = []
        self._exclude_functions = []

    @property
    def source(self):
        return self._psp

    @source.setter
    def source(self, value):
        self._psp = value
        _, trailing = os.path.split(value)
        self._pkgname = trailing.strip()

    @property
    def docs(self):
        return self._dsp

    @docs.setter
    def docs(self, value):
        self._dsp = value

    @property
    def exclude_files(self):
        return self._exclude_files

    @exclude_files.setter
    def exclude_files(self, value):
        files = []
        patterns = []

        for val in value:
            if val.endswith(".py") or val.endswith(".pyx"):
                files.append(val)
            else:
                patterns.append(re.compile("%s" % val))

        self._exclude_files = files, patterns

    @property
    def exclude_pkgs(self):
        return self._exclude_pkgs

    @exclude_pkgs.setter
    def exclude_pkgs(self, value):
        pkgs = []
        patterns = []

        for val in value:
            patterns.append(re.compile("%s" % val))

        self._exclude_pkgs = pkgs, patterns

    @property
    def exclude_classes(self):
        return self._exclude_classes

    @exclude_classes.setter
    def exclude_classes(self, value):
        self._exclude_classes = value

    @property
    def exclude_enums(self):
        return self._exclude_enums

    @exclude_enums.setter
    def exclude_enums(self, value):
        self._exclude_enums = value

    @property
    def exclude_dataclasses(self):
        return self._exclude_dataclasses

    @exclude_dataclasses.setter
    def exclude_dataclasses(self, value):
        self._exclude_dataclasses = value

    @property
    def exclude_methods(self):
        return self._exclude_methods

    @exclude_methods.setter
    def exclude_methods(self, value):
        m = []
        for val in value:
            owner = None
            if "." in val:
                owner, val = val.split(".")

            m.append((val, owner))

        self._exclude_methods = m

    @property
    def exclude_functions(self):
        return self._exclude_functions

    @exclude_functions.setter
    def exclude_functions(self, value):
        self._exclude_functions = value

    @property
    def package_name(self):
        return self._pkgname


CONFIG = Config()



class ObjectInfo(object):
    def __init__(self, name, file, streampos):
        self.__name = name
        self.__file = file
        self.__streampos = streampos

        if name.startswith("__"):
            self.__access = "private"
        elif name.startswith("_"):
            self.__access = "protected"
        else:
            self.__access = "public"

    @property
    def name(self):
        return self.__name

    @property
    def is_private(self):
        return self.__access == "private"

    @property
    def is_protected(self):
        return self.__access == "protected"

    @property
    def is_public(self):
        return self.__access == "public"

    @property
    def owning_file(self):
        return self.__file

    @property
    def streampos(self):
        return self.__streampos


class ClassInfo(ObjectInfo):
    def __init__(self, name, file, streampos, abstract):
        super().__init__(name, file, streampos)
        self.__abstract = abstract

    @property
    def is_abstract(self):
        return self.__abstract


class EnumInfo(ObjectInfo):
    def __init__(self, name, file, streampos):
        super().__init__(name, file, streampos)


class DataClassInfo(ObjectInfo):
    def __init__(self, name, file, streampos):
        super().__init__(name, file, streampos)


class MethodInfo(ObjectInfo):
    def __init__(self, name, file, streampos, mtype, owner):
        super().__init__(name, file, streampos)
        self.__mtype = mtype
        self.__owner = owner

    @property
    def mtype(self):
        return self.__mtype

    @property
    def owner(self):
        return self.__owner


class FunctionInfo(ObjectInfo):
    def __init__(self, name, file, streampos):
        super().__init__(name, file, streampos)



import argparse
import difflib
import enum
import os
import sys

import yaml

from . import CONFIG, info_print
from . import __version__

from .cleaner import clean_api_docs
from .loader import file_mappings, mapping_tostr
from .parsing import Parser
from .writer import write


def write_config_file():
    cwd = os.getcwd()

    conf = os.path.join(cwd, ".reslate-config.yaml")
    if os.path.isfile(conf):
        os.remove(conf)

    root_name = os.path.normpath(cwd).split(os.sep)[-1].split("_")[0]
    dirs_in_cwd = list(filter(lambda x: os.path.isdir(x), os.listdir(cwd)))

    closest_to_rootname = difflib.get_close_matches(root_name, dirs_in_cwd, n=1)
    if not closest_to_rootname:
        info_print(
            "WARNING: Unable to determine package_source_path. Please set this "
            "field manually in .reslate-config.yaml",
            True,
        )
        relative_source_path = ""
    else:
        relative_source_path = closest_to_rootname[0]

    closest_to_docs = difflib.get_close_matches("docs", dirs_in_cwd, n=1)
    if not closest_to_rootname:
        info_print(
            "WARNING: Unable to determine docs_source_path. Please set this "
            "field manually in .reslate-config.yaml",
            True,
        )
        relative_docs_src_path = ""
    else:
        relative_docs_path = closest_to_docs[0]

        docs_path = os.path.join(cwd, relative_docs_path)

        dirs_in_docs = list(
            filter(
                lambda x: os.path.isdir(os.path.abspath(os.path.join(docs_path, x))),
                os.listdir(docs_path),
            )
        )
        if "source" in dirs_in_docs:
            relative_docs_src_path = os.path.join(relative_docs_path, "source")
        elif "src" in dirs_in_docs:
            relative_docs_src_path = os.path.join(relative_docs_path, "src")
        else:
            relative_docs_src_path = relative_docs_path

    contents = r"""
# path to package source code relative to package root
package_source_path: %s

# path to package documentation source relative to package root
docs_source_path: %s

# files and patterns to exclude from documenting
exclude:
    files: ['^_', '^\.', 'version', 'test']
    subpkgs: ['^_', '^\.', 'version', 'test']

    classes: []
    enums: []
    dataclasses: []
    methods: []
    functions: []
    """ % (
        relative_source_path,
        relative_docs_src_path,
    )

    with open(conf, "w") as f:
        f.write(contents.strip() + "\n")

    info_print(f"Configuration file: {conf} written successfully.", True)


def find_config_file():
    directory = os.getcwd()

    conf_file = os.path.join(directory, ".reslate-config.yaml")

    while not os.path.isfile(conf_file) and directory:
        directory, _ = os.path.split(directory)
        conf_file = os.path.join(directory, ".reslate-config.yaml")

    if not directory:
        info_print("ERROR: Unable to find a .reslate-config.yaml file!", True)
        return None, None

    with open(conf_file, "r") as f:
        try:
            return yaml.safe_load(f), directory
        except yaml.YAMLError as ex:
            info_print(
                f"ERROR: Unable to load .reslate-config.yaml file, the "
                f"following yaml load error occurred:\n    {str(ex)}",
                True,
            )
            return None, directory


def _make_parser():
    parser = argparse.ArgumentParser(
        description="Generate and update API documentation."
    )

    parser.add_argument(
        "sources",
        nargs="*",
        help="Source files of package to generate / update API "
        "documentation pages from.",
    )

    parser.add_argument(
        "--init",
        dest="initialise",
        action="store_true",
        help="Initialise reslate for the current working directory. Note that "
        "the current working directory should be the root directory of your "
        "local copy of your package repository.",
        default=False,
    )
    parser.add_argument(
        "--clean", dest="clean", action="store_true", default=False,
    )

    parser.add_argument(
        "-V",
        "--version",
        help="Current version of reslate.",
        action="version",
        version="reslate v%s" % __version__,
    )
    parser.add_argument("-v", dest="verbose", action="store_true", default=False)

    return parser


def main():
    parser = _make_parser()

    # parse the args provided into a Namespace
    args = parser.parse_args()

    if args.initialise:
        info_print("Attempting to write .reslate-config.yaml file", True)
        write_config_file()

    config, proj_root = find_config_file()
    if config is None:
        return -1

    CONFIG.source = os.path.join(
        proj_root, os.path.normpath(config["package_source_path"])
    )
    info_print(f"Using package source path: {CONFIG.source}", args.verbose)
    info_print(f"Determined package name: {CONFIG.package_name}", args.verbose)

    CONFIG.docs = os.path.join(proj_root, os.path.normpath(config["docs_source_path"]))
    info_print(f"Using docs source path: {CONFIG.docs}", args.verbose)

    if args.clean:
        clean_api_docs()

    exclude = config.get("exclude", {})
    CONFIG.exclude_files = exclude.get("files", [])
    CONFIG.exclude_pkgs = exclude.get("subpkgs", [])
    CONFIG.exclude_classes = exclude.get("classes", [])
    CONFIG.exclude_enums = exclude.get("enums", [])
    CONFIG.exclude_dataclasses = exclude.get("dataclasses", [])
    CONFIG.exclude_methods = exclude.get("methods", [])
    CONFIG.exclude_functions = exclude.get("functions", [])

    sources = []
    for src in args.sources:
        src = os.path.abspath(src)
        if os.path.isfile(src) and (src.endswith(".py") or src.endswith(".pyx")):
            sources.append(src)
        elif os.path.isdir(src):
            for root, _, files in os.walk(src):
                for f in files:
                    if f.endswith(".py") or f.endswith(".pyx"):
                        af = os.path.abspath(os.path.join(root, f))
                        sources.append(af)
        else:
            info_print(
                f"WARNING: Ignoring file or directory: {src} as it does not exist ",
                True,
            )

    if not sources:
        info_print("No source files given, exiting.", args.verbose)
        return 0

    subpkg_doc_files, submod_doc_files = file_mappings(sources)

    if args.verbose:
        info_print(
            f"Sub-package source -> doc file mappings:\n"
            f"{mapping_tostr(subpkg_doc_files)}\n",
            args.verbose,
        )
        info_print(
            f"Sub-module source -> doc file mappings:\n"
            f"{mapping_tostr(submod_doc_files)}\n",
            args.verbose,
        )

    map_sfi_to_docfile = {}
    for src, docfile in submod_doc_files.items():
        p = Parser(src)
        map_sfi_to_docfile[p.parse()] = docfile

    return write(subpkg_doc_files, map_sfi_to_docfile)


if __name__ == "__main__":
    sys.exit(main())



import os
from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


def load_requirements():
    contents = read("requirements.txt")
    return [line.strip() for line in contents.splitlines()]


CLASSIFIERS = [
    "Development Status :: 3 - Alpha",
    "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Topic :: Software Development :: Documentation",
    "Topic :: Software Development :: Version Control :: Git",
    "Topic :: Documentation",
    "Topic :: Documentation :: Sphinx",
]

KEYWORDS = "documentation git pre-commit"

setup(
    name="reslate",
    use_scm_version={"write_to": "reslate/version.py"},
    author="Samuel Rowlinson",
    author_email="sjrowlinson@virginmedia.com",
    description=(
        "A command-line tool to generate and update Sphinx-based API documentation pages."
    ),
    url="https://gitlab.com/sjrowlinson/reslate",
    license="GPL",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=load_requirements(),
    setup_requires=["setuptools_scm"],
    python_requires=">=3.6",
    classifiers=CLASSIFIERS,
    keywords=KEYWORDS,
    project_urls={"Source": "https://gitlab.com/sjrowlinson/reslate",},
    package_data={"reslate": ["templates/templates/*.rst",]},
    include_package_data=True,
    entry_points={"console_scripts": ["reslate = reslate.__main__:main"]},
)
